@@ -43,6 +43,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <unbound.h>
+#include <assert.h>
 
 #include "context.h"
 #include "types-internal.h"
@@ -416,6 +417,7 @@ getdns_context_create_with_extended_memory_functions(
     if (!result) {
         return GETDNS_RETURN_GENERIC_ERROR;
     }
+    result->processing = 0;
     result->destroying = 0;
     result->my_mf.mf_arg         = userarg;
     result->my_mf.mf.ext.malloc  = malloc;
@@ -437,8 +439,10 @@ getdns_context_create_with_extended_memory_functions(
 
 
     result->resolution_type = GETDNS_RESOLUTION_RECURSING;
-    if(create_default_namespaces(result) != GETDNS_RETURN_GOOD)
+    if(create_default_namespaces(result) != GETDNS_RETURN_GOOD) {
+		getdns_context_destroy(result);
 		return GETDNS_RETURN_GENERIC_ERROR;
+    }
 
     result->timeout = 5000;
     result->follow_redirects = GETDNS_REDIRECTS_FOLLOW;
@@ -469,6 +473,7 @@ getdns_context_create_with_extended_memory_functions(
     result->dns_transport = GETDNS_TRANSPORT_UDP_FIRST_AND_FALL_BACK_TO_TCP;
     result->limit_outstanding_queries = 0;
     result->has_ta = priv_getdns_parse_ta_file(NULL, NULL);
+    result->return_dnssec_status = GETDNS_EXTENSION_FALSE;
     if (!result->outbound_requests ||
         !result->timeouts_by_id ||
         !result->timeouts_by_time) {
@@ -534,6 +539,12 @@ getdns_context_destroy(struct getdns_context *context)
     if (context == NULL) {
         return;
     }
+    // If being destroyed during getdns callback,
+    // fail via assert
+    assert(context->processing == 0);
+    if (context->destroying) {
+        return ;
+    }
     context->destroying = 1;
     cancel_outstanding_requests(context, 1);
     getdns_extension_detach_eventloop(context);
@@ -570,7 +581,6 @@ getdns_context_destroy(struct getdns_context *context)
         GETDNS_FREE(context->my_mf, context->timeouts_by_time);
 
     GETDNS_FREE(context->my_mf, context);
-    return;
 }               /* getdns_context_destroy */
 
 /*
@@ -1203,11 +1213,13 @@ getdns_cancel_callback(struct getdns_context *context,
     getdns_transaction_t transaction_id)
 {
     RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
+    context->processing = 1;
     getdns_return_t r = getdns_context_cancel_request(context, transaction_id, 1);
     if (context->extension) {
         context->extension->request_count_changed(context,
             context->outbound_requests->count, context->extension_data);
     }
+    context->processing = 0;
     return r;
 } /* getdns_cancel_callback */
 
@@ -1410,7 +1422,25 @@ getdns_context_clear_outbound_request(getdns_dns_req * req)
     return GETDNS_RETURN_GOOD;
 }
 
+getdns_return_t
+getdns_context_request_timed_out(struct getdns_dns_req
+    *req) {
+    getdns_context* context = req->context;
+    getdns_transaction_t trans_id = req->trans_id;
+    getdns_callback_t cb = req->user_callback;
+    void *user_arg = req->user_pointer;
 
+    /* cancel the req - also clears it from outbound and cleans up*/
+    getdns_context_cancel_request(context, trans_id, 0);
+    context->processing = 1;
+    cb(context, GETDNS_CALLBACK_TIMEOUT, NULL, user_arg, trans_id);
+    context->processing = 0;
+    if (context->extension) {
+        context->extension->request_count_changed(context,
+            context->outbound_requests->count, context->extension_data);
+    }
+    return GETDNS_RETURN_GOOD;
+}
 
 char *
 getdns_strdup(const struct mem_funcs *mfs, const char *s)
@@ -1505,12 +1535,16 @@ getdns_context_get_num_pending_requests(struct getdns_context* context,
 /* process async reqs */
 getdns_return_t getdns_context_process_async(struct getdns_context* context) {
     RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
+    context->processing = 1;
     if (ub_poll(context->unbound_ctx)) {
         if (ub_process(context->unbound_ctx) != 0) {
             /* need an async return code? */
+            context->processing = 0;
             return GETDNS_RETURN_GENERIC_ERROR;
         }
     }
+    // reset the processing flag
+    context->processing = 0;
     if (context->extension != NULL) {
         /* no need to process timeouts since it is delegated
          * to the extension */
@@ -1524,7 +1558,8 @@ getdns_return_t getdns_context_process_async(struct getdns_context* context) {
         return GETDNS_RETURN_GENERIC_ERROR;
     }
     ldns_rbnode_t* next_timeout = ldns_rbtree_first(context->timeouts_by_time);
-    while (next_timeout) {
+    getdns_return_t r = GETDNS_RETURN_GOOD;
+    while (next_timeout && r == GETDNS_RETURN_GOOD) {
         getdns_timeout_data_t* timeout_data = (getdns_timeout_data_t*) next_timeout->data;
         if (timeout_cmp(timeout_data, &key) > 0) {
             /* no more timeouts need to be fired. */
@@ -1541,10 +1576,10 @@ getdns_return_t getdns_context_process_async(struct getdns_context* context) {
         }
 
         /* fire the timeout */
-        timeout_data->callback(timeout_data->userarg);
+        r = timeout_data->callback(timeout_data->userarg);
     }
 
-    return GETDNS_RETURN_GOOD;
+    return r;
 }
 
 typedef struct timeout_accumulator {
@@ -1577,19 +1612,28 @@ cancel_outstanding_requests(struct getdns_context* context, int fire_callback) {
 getdns_return_t
 getdns_extension_detach_eventloop(struct getdns_context* context)
 {
-    RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
-    getdns_return_t r = GETDNS_RETURN_GOOD;
-    if (context->extension) {
-        /* cancel all outstanding requests */
-        cancel_outstanding_requests(context, 1);
-        r = context->extension->cleanup_data(context, context->extension_data);
-        if (r != GETDNS_RETURN_GOOD) {
-            return r;
-        }
-        context->extension = NULL;
-        context->extension_data = NULL;
-    }
-    return r;
+	RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
+	getdns_return_t r = GETDNS_RETURN_GOOD;
+	if (context->extension) {
+		/* When called from within a callback, do not execute pending
+		 * context destroys.
+		 * The (other) callback handler will handle it.
+		 *
+		 * ( because callbacks occur in cancel_outstanding_requests,
+		 *   and they may destroy the context )
+		 */
+		context->processing = 1;
+		/* cancel all outstanding requests */
+		cancel_outstanding_requests(context, 1);
+		r = context->extension->cleanup_data(context,
+		    context->extension_data);
+		if (r == GETDNS_RETURN_GOOD) {
+			context->extension = NULL;
+			context->extension_data = NULL;
+		}
+		context->processing = 0;
+	}
+	return r;
 }
 
 getdns_return_t
@@ -1783,7 +1827,7 @@ getdns_context_get_api_information(getdns_context* context) {
 getdns_return_t
 getdns_context_set_return_dnssec_status(getdns_context* context, int enabled) {
     RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
-    if (enabled != GETDNS_EXTENSION_TRUE ||
+    if (enabled != GETDNS_EXTENSION_TRUE &&
         enabled != GETDNS_EXTENSION_FALSE) {
         return GETDNS_RETURN_INVALID_PARAMETER;
     }
@@ -1791,4 +1835,19 @@ getdns_context_set_return_dnssec_status(getdns_context* context, int enabled) {
     return GETDNS_RETURN_GOOD;
 }
 
+getdns_return_t
+getdns_context_set_use_threads(getdns_context* context, int use_threads) {
+    RETURN_IF_NULL(context, GETDNS_RETURN_INVALID_PARAMETER);
+    if (context->resolution_type_set != 0) {
+        /* already setup */
+        return GETDNS_RETURN_CONTEXT_UPDATE_FAIL;
+    }
+    int r = 0;
+    if (use_threads)
+        r = ub_ctx_async(context->unbound_ctx, 1);
+    else
+        r = ub_ctx_async(context->unbound_ctx, 0);
+    return r == 0 ? GETDNS_RETURN_GOOD : GETDNS_RETURN_CONTEXT_UPDATE_FAIL;
+}
+
 /* context.c */
