@@ -31,6 +31,10 @@
 #include <getdns/getdns.h>
 #include <sys/time.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* Enable the return_dnssec_status extension on every request.
    value is either GETDNS_EXTENSION_TRUE or GETDNS_EXTENSION_FALSE
    returns GETDNS_RETURN_GOOD on success or GETDNS_RETURN_INVALID_PARAMETER
@@ -55,9 +59,11 @@ int getdns_context_fd(getdns_context* context);
 /* process async reqs */
 getdns_return_t getdns_context_process_async(getdns_context* context);
 
+/* tells underlying unbound to use background threads or fork */
+getdns_return_t getdns_context_set_use_threads(getdns_context* context, int use_threads);
 
 /* extensions */
-typedef void (*getdns_timeout_callback) (void* userarg);
+typedef getdns_return_t (*getdns_timeout_callback) (void* userarg);
 
 /* context timeout data */
 typedef struct getdns_timeout_data {
@@ -115,5 +121,8 @@ getdns_context_get_extension_data(struct getdns_context* context);
 getdns_return_t
 getdns_extension_detach_eventloop(struct getdns_context* context);
 
+#ifdef __cplusplus
+}
+#endif
 
 #endif
